#include "colorlabel.h"
#include "ui_colorlabel.h"
#include "state.hpp"
#include "stateColor.hpp"
#include "alphabet.hpp"
#include <stdlib.h>
#include <string>
#include <QListWidget>

ColorLabel::ColorLabel(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::ColorLabel)
{
    ui->setupUi(this);
    connect(ui->generate_button, SIGNAL(clicked()), this, SLOT(generateState(a,state_label->text(),red_slider->value(),green_slider->value(),blue_slider->value())));
    connect(ui->random_button, SIGNAL(clicked()), this, SLOT(randomGenerator(a,state_label->text())));
    connect(ui->daltonian_button, SIGNAL(clicked()), this, SLOT(daltonianMode(a,nbStates)));
}

ColorLabel::~ColorLabel()
{
    delete ui;
}

void ColorLabel::generateState(Alphabet a, QString name_state, unsigned int r, unsigned int g, unsigned int b){
    std::string string_name_state = name_state.toStdString(); // On passe notre QString en string pour pouvoir utiliser le constructeur
    state* newstate = new state(stateColor(r,g,b),string_name_state); // Création de notre nouvel état
    a.newEtat(*newstate); // Ajout du nouvel état dans l'alphabet
    ui->state_list->addItem(name_state); // Ajout du nouvel état dans la liste
}


void ColorLabel::randomGenerator(Alphabet a, QString name_state){
    std::string string_name_state = name_state.toStdString();
    unsigned int r = rand() % 255; // On génère aléatoirement nos couleurs
    unsigned int g = rand() % 255;
    unsigned int b = rand() % 255;
    state* newstate = new state(stateColor(r,g,b),string_name_state); // Création de notre nouvel état
    a.newEtat(*newstate); // Ajout du nouvel état dans l'alphabet
    ui->state_list->addItem(name_state); // Ajout du nouvel état dans la liste
}


void ColorLabel::daltonianMode(Alphabet a, unsigned int nbStates){
    ui->state_list->clear(); // On vide toujours la liste quand on active le mode Daltonien

    // Création des états à partir du constructeur
    state* black = new state(stateColor(0,0,0),"Black");
    state* white = new state(stateColor(255,255,255),"White");
    state* blue = new state(stateColor(0,0,255),"Blue");
    state* pink = new state(stateColor(255,0,255),"Pink");
    state* red = new state(stateColor(255,0,0),"Red");
    state* green = new state(stateColor(0,255,0),"Green");
    state* orange = new state(stateColor(255,127,0),"Orange");
    state* grey = new state(stateColor(127,127,127),"Grey");

    // Ajout des états dans la liste
    ui->state_list->addItem(QString::fromStdString(black->getStateLabel()));
    ui->state_list->addItem(QString::fromStdString(white->getStateLabel()));
    ui->state_list->addItem(QString::fromStdString(blue->getStateLabel()));
    ui->state_list->addItem(QString::fromStdString(pink->getStateLabel()));
    ui->state_list->addItem(QString::fromStdString(red->getStateLabel()));
    ui->state_list->addItem(QString::fromStdString(green->getStateLabel()));
    ui->state_list->addItem(QString::fromStdString(orange->getStateLabel()));
    ui->state_list->addItem(QString::fromStdString(grey->getStateLabel()));

    // Ajout des états dans l'Alphabet
    a.newEtat(*black);
    a.newEtat(*white);
    a.newEtat(*blue);
    a.newEtat(*pink);
    a.newEtat(*red);
    a.newEtat(*green);
    a.newEtat(*orange);
    a.newEtat(*grey);

}
