/**
\file simulation.hpp
\brief Simulation

Cette classe représente un automate cellulaire dans le temps.

**/

#ifndef SIMULATION_HPP
#define SIMULATION_HPP

#include "history.h"
#include "grid.h"
#include "automaton.hpp"

class Simulation {
private:
    bool canRun;
    int time;
    Automaton automaton;
    Grid startGrid;
    History hist;

public:
    Simulation();
    virtual ~Simulation();

    //! \brief Définit la règle de voisinage
    //!
    //! La règle est passée à l'automate de la simulation qui se charge de la destruction de ses règles. Aucune copie n'est faite.
    //! \param NR La nouvelle règle de voisinage
    void setNeighborhoodRule(NeighborhoodRule*);

    //! \brief Définit la règle de transition
    //!
    //! La règle est passée à l'automate de la simulation qui se charge de la destruction de ses règles. Aucune copie n'est faite.
    //! \param TR La nouvelle règle de transition
    void setTransitionRule(TransitionRule*);



    //! \brief Définit l'aplhabet de l'automate de la simulation
    //!
    //! La grille de départ et la grille de l'automate sont vidées
    //! \param A Le nouvel alphabet
    void setAlphabet(const Alphabet&);

    //! \brief Accesseur sur l'alphabet de l'automate de la simulation
    //!
    //! \return L'alphabet de l'automate
    const Alphabet& getAlphabet() const {return automaton.getAlphabet();}



    //! \brief Définit la grille
    //!
    //! Si la simulation est lancée, cela ne modifie pas la grille de départ. Cela définit la grille de départ dans le cas contraire.
    //! \param grid La nouvelle grille
    void setGrid(const Grid&);

    //! \brief Retourne la grille actuelle
    //!
    //! \return Grille de l'automate
    const Grid& getGrid() const;
    //void setCell(const Coord&, unsigned int);
    //void resize(size_t,size_t);



    //! \brief Remet la simulation dans son état de départ
    void reset();

    //! \brief Fait avancer la simulation d'un pas
    void step();

    //! \brief Fais reculer la simulation d'un pas si possible
    void back();
};

#endif // SIMULATION_HPP
