#include "neighborhood.hpp"

unsigned int Neighborhood::getNb(unsigned int s) const
{
    unsigned int res = 0;

    for (auto it = neighborPositions.begin(); it != neighborPositions.end(); ++it) {
        if (it->second == s) {
            res++;
        }
    }

    return res;
}

unsigned int Neighborhood::getAt(Coord c) const
{
    if (!neighborPositions.count(c))
        throw NeighborhoodException("Error Neighborhood::getAt");

    return neighborPositions.at(c);
}

Neighborhood Neighborhood::flip_vertically() const
{
    Neighborhood n;
    for (const auto& val : neighborPositions)
    {
        n.addNeighbor(Coord{val.first.x, -val.first.y}, val.second);
    }
    return n;
}

Neighborhood Neighborhood::flip_horizontally() const
{
    Neighborhood n;
    for (const auto& val : neighborPositions)
    {
        n.addNeighbor(Coord{-val.first.x, val.first.y}, val.second);
    }
    return n;
}

Neighborhood Neighborhood::rotate90() const
{
    Neighborhood n;
    for (const auto& val : neighborPositions)
    {
        n.addNeighbor(Coord{-val.first.y, val.first.x}, val.second);
    }
    return n;
}

bool Neighborhood::isUnique(Coord c) const
{
    return neighborPositions.count(c) == 0;
}
